#include <stdio.h>
#include <omp.h>

#ifndef LINESIZE
#define LINESIZE 1
#endif

int main(int argc, char* argv[]) {
    int n = 100 << 20;

    double* x = (double*)malloc(n * sizeof(double));
    double* y = (double*)malloc(n * sizeof(double));

    for (int i = 0; i < n; ++i) {
        x[i] = (double)(i);
        y[i] = (double)(-i);
    }

    double local_sum[LINESIZE*omp_get_max_threads()];
    double sum = 0.0;

    double elapsed = omp_get_wtime();
    
    #pragma omp parallel shared(sum)
    {
        int tid = omp_get_thread_num();
        local_sum[tid*LINESIZE] = 0.0;

        #pragma omp for
        for (int i = 0; i < n; ++i)
            local_sum[tid*LINESIZE] += 0.5 * x[i] + y[i];
    
        #pragma omp atomic
        sum += local_sum[tid*LINESIZE];
    }

    elapsed = omp_get_wtime() - elapsed;

    printf("Elapsed time = %.6lf s\n", elapsed);
    

    free(x); free(y);

    return 0;
}